<?php

namespace Drupal\givebutter\Form;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Url;

/**
 * Configure Givebutter settings for this site.
 */
class GivebutterSettingsForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'givebutter.settings';

  /**
   * A string map, from config key to form element key.
   *
   * @var string[]
   */
  const FORM_CONFIG_KEY_MAP = [
    'script_acct' => 'script.acct',
    'visibility_enable' => 'visibility.enable',
    'visibility_request_path_mode' => 'visibility.request_path_mode',
    'visibility_request_path_pages' => 'visibility.request_path_pages',
  ];

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'givebutter_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);

    $intro_message = $this->t('Visit your <a href=":givebutterWebsite" target="_blank">Givebutter dashboard</a> to find your account code or install widgets instruction which will include it.', [
      ':givebutterWebsite' => 'https://dashboard.givebutter.com/',
    ]);
    $form['intro'] = [
      '#markup' => '<p>' . $intro_message . '</p>',
    ];


    $form['script_acct'] = [
      '#default_value' => $config->get('script.acct'),
      '#type' => 'textfield',
      '#title' => $this->t('Account ID'),
      '#description' => $this->t('Found after `acct` in the Givebutter script, this is your account identifier.'),
      '#required' => TRUE,
    ];

    $form['visibility_enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable'),
      '#description' => $this->t('Global toggle for enabling/disabling tracking.'),
      '#default_value' => $config->get('visibility.enable'),
    ];

    $form['visibility_request_path_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Add script to specific pages'),
      '#options' => [
        $this->t('Disabled (show on all non-admin pages)'),
        $this->t('Every page except the listed pages'),
        $this->t('The listed pages only'),
      ],
      '#default_value' => $config->get('visibility.request_path_mode'),
      '#description' => $this->t('By default if enabled the Givebutter header script will be added to all non-admin pages.  This can be restricted here but adding it to admin pages is not supported.')
    ];

    $visibility_request_path_pages = $config->get('visibility.request_path_pages');
    $title = $this->t('Pages');
    $description = $this->t("Specify pages by using their paths. Enter one path per line.
     The '*' character is a wildcard. Example paths are %blog for the blog page and
     %blog-wildcard for every personal blog. %front is the front page.", [
       '%blog' => '/blog',
       '%blog-wildcard' => '/blog/*',
       '%front' => '<front>',
     ]);

    $form['tracking']['page_visibility_settings']['visibility_request_path_pages'] = [
      '#type' => 'textarea',
      '#title' => $title,
      '#title_display' => 'invisible',
      '#default_value' => !empty($visibility_request_path_pages) ? $visibility_request_path_pages : '',
      '#description' => $description,
      '#rows' => 10,
      '#states' => [
        'visible' => [
          ':input[name="visibility_request_path_mode"]' => ['!value' => '0'],
        ],
      ],
    ];

    $this->disableOverriddenElements($form);

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    // Verify that every path is prefixed with a slash, but do not check for
    // slashes if no paths configured.
    if ($form_state->getValue('visibility_request_path_mode') != 2
      && !empty($form_state->getValue('visibility_request_path_pages'))) {

      $pages = preg_split('/(\r\n?|\n)/', $form_state->getValue('visibility_request_path_pages'));
      foreach ($pages as $page) {
        if (strpos($page, '/') !== 0 && $page !== '<front>') {
          $msg = $this->t('Path "@page" not prefixed with slash.', ['@page' => $page]);
          $form_state->setErrorByName('visibility_request_path_pages', $msg);
          // Drupal forms show one error only.
          break;
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Retrieve the configuration.
    $config = $this->configFactory->getEditable(static::SETTINGS);

    // Set the submitted configuration setting.
    foreach (static::FORM_CONFIG_KEY_MAP as $element_key => $config_key) {
      $config->set($config_key, $form_state->getValue($element_key));
    }
    $config->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Recursively disable elements whose config values are overridden.
   */
  protected function disableOverriddenElements(array &$form): void {
    $config = $this->configFactory->get(static::SETTINGS);

    foreach (Element::children($form) as $element_key) {
      $config_key = static::FORM_CONFIG_KEY_MAP[$element_key] ?? NULL;

      if ($config_key !== NULL && $config->get($config_key) !== $config->getOriginal($config_key, FALSE)) {
        $form[$element_key]['#disabled'] = TRUE;
        $form[$element_key]['#description'] = $this->t('This config cannot be changed because it is overridden.');
      }

      if (is_array($form[$element_key])) {
        $this->disableOverriddenElements($form[$element_key]);
      }
    }
  }

}
